####################
# version 2.0
####################


################################################################################
#                                                                              #
# NAME Bio.jaspar
# DESC Utilities for reading JASPAR DB
#                                                                              #
################################################################################
"""
:NAME:
Bio.jaspar

:DESC:
Utilities for reading JASPAR files

methods:
    read_annotation
    read_matrix
    read_matrices
    extract_sites

"""
# IMPORTS
#-------------------------------------------------------------------------------

import os
import re
import sys
import input_checker as ic
import Core.UI.cli as cli
#-------------------------------------------------------------------------------
# IMPORTS


# CODE
#-------------------------------------------------------------------------------

annotfilename='matrix_list.txt' # file with the annotations of the psfm


def read_annotation(filename):
    """
    Read the annotation file
    
    filename -- a text file with the annotation of Jaspar files
    
    return   -- a dictionnary with for each matrix its annotations
    """
    f = open(filename)
    annotations = {}
    
    for line in f:
        split = line.strip().split(';')
        first=split[0]
	split_first=first.split('\t')
	id=split_first[0]

	features = annotations.setdefault(id, {})
	features["name"]=split_first[2]
	features["class"]=split_first[3]
	others=split[1:]

	for i in others:
		split_others= i.split(' ')
		key=split_others[1]
		value=split_others[2][1:-1]
		
       	        features[key] = value
   	
    return annotations


def read_matrix(filename):
    """
    read an pfm files and return the matrix inside
    
    filename -- a pfm file with a Jaspar matrix
    
    return   -- an array with the matrix
    """
    
    m = []
    strike=0
    len_line=0
    
    for line in open(filename):
    
        if ic.verif_jaspar(line) == False:
            cli.error('a Jaspar matrix is not in correct format: '+filename)
            sys.exit(1)

        if strike==0:
            len_line=len(line)
        if len(line) != len_line:
            cli.error('a Jaspar matrix is malformed: '+ filename)
            sys.exit(1)
            
        m += [ map(float, line.split()) ]
        strike +=1
        
    return m

def read_matrices(annotationFilename, matrixDirectory, extension='.pfm'):
    """
    read an annotation file and all the pfm files corresponding to it.
    
    annotationFilename -- the file with the annotation of the Jaspar matrix files
    
    matrixDirectory    -- the directory where to find all the Jaspar files (annotations + pfm)
    
    extension          -- the extension of Jaspar matrix files
    
    return             -- a list of matrix
    """
    matrices = {}
    annotations = read_annotation(annotationFilename)
    
    for id in annotations:
        matrix = read_matrix(os.path.join(matrixDirectory, id+extension))
        matrices[id] = dict(matrix=matrix, features=annotations[id])
    return matrices


def extract_sites(filename, core=True):
    """Extract sites from JASPAR *.sites

    filenanme -- name of JASPAR file
    
    core      -- keep only core (default=True)
    
    return    -- list of sites (core) ['CCATATATAG', 'CCATATATAG', ...]
    """
    sites = []
    lines = open(filename).readlines()
    for i in range(0, len(lines), 2):
        if lines[i].startswith('>'):
            site = lines[i+1]
            if core:
                site = re.findall(r'([ACGT]+)', site)[0]
            sites.append(site.upper())
        else:
            break
    return sites


#-------------------------------------------------------------------------------
# CODE
