####################
# version 2.0
####################


################################################################################
#                                                                              #
# NAME Core.IO.cache 
# DESC Cached IO
#                                                                              #
################################################################################
"""
:NAME:
Core.IO.cache

:DESC:
Cached IO


"""
# IMPORTS
#-------------------------------------------------------------------------------
import os
import random
#-------------------------------------------------------------------------------
# IMPORTS

# CODE
#-------------------------------------------------------------------------------

class ObjectPropertyCache:
    """Cache in file properties of an object
    
    """
    
    def __init__(self, dir, fidx=lambda x: id(x), rcall=lambda x:x, wcall=None, mode='rw'):
        """
        dir   -- directory where to store files
        fidx  -- function that index objects used to convert object to filename
        rcall -- apply this function to filename when reading
        wcall -- write to file result of this function applied to object

        by default rcall return the filename
        """
        
        self.dir   = dir
        self.mode  = mode
        self.fidx  = fidx
        self.rcall = rcall
        self.wcall = wcall
        
        if not os.path.exists(self.dir) and 'w' in self.mode:
            os.mkdir(self.dir)
        
    def __getitem__(self, obj):
        """
        return rcall apply to obj cache filename
        """
        if not os.path.exists(self.fidx(obj)):
            self.write(obj)
        return self.read(obj) 
    
    def read(self, obj):
        path = os.path.join(self.dir, self.fidx(obj))
        return self.rcall(path)        
    
    def write(self, obj):
        path = os.path.join(self.dir, self.fidx(obj))
        f = open(path, 'w')
        f.write(self.wcall(obj))
        f.close()
            


#-------------------------------------------------------------------------------
# CODE
