####################
# version 2.0
####################


################################################################################
#                                                                              #
# NAME Types.interval
# DESC interval
#                                                                              #
################################################################################
"""
:NAME:
Types.interval

:DESC:
A interval can be simply defined by a tuple (a,b)

"""
# IMPORTS
#-------------------------------------------------------------------------------
import list
#-------------------------------------------------------------------------------
# IMPORTS

# CODE
#-------------------------------------------------------------------------------

def is_included(x, y):
    """
    x -- tuple (i,j)
    y -- tuple (i',j')
    return True if interval x is included in y
    """
    return x[0] >= y[0] and x[1] <= y[1]

def is_overlapping(x, y):
    """
    x -- tuple (i,j)
    y -- tuple (i',j')
    return True if interval x overlap interval y
    """
    return (x[0] >= y[0] and x[0] <= y[1]) or (x[1] >= y[0] and x[1] <= y[1])

def intersection(x, y):
    """
    x -- tuple (i,j)
    y -- tuple (i',j')
    return overlapping interval or None
    """
    a = max(x[0], y[0])
    b = min(x[1], y[1])

    if b >= a:
        return (a,b)
    else:
        return None

def find_intervals(l, n=2, min=0.0, max=1.0):
    """cut a list of values into n intervals that contain the same number of values
    
    l   -- list of values
    n   -- number of intervals to guess
    min -- lower limit
    max -- upper limit
    
    return list of tuple corresponding to intervals found
    """
    intervals = []
    a = min
    chunks = types.list.chunked(gc, int(len(gc)/float(gcCategories))+1)
    l = types.list.sorted(l)
    for i in range(0, len(l), l / int(n)):
        b = l[i]
        intervals.append( (a,b) )
        a = b
    intervals.append( (a,max))
    return intervals

def find(v, l):
    """
    find in which interval is v

    v -- value
    l -- list of tuple corresponding to intervals

    return tuple or None
    
    """

    for i in l:
        if v >= i[0] and v <= i[1]:
            return i
    return None


def segment(location, step):
    l = []
    for i in range(location[0], location[1]+1, step):
        l.append( (i, min(location[1], i+step-1)) )
        
    return l

def cutting(intervals, location):
    """ Find intervals to use
    intervals -- list of invertal (list of tuple)
    location  -- tuple
    
    return list of tuple ( (interval, freq) ) where freq is interval contribution for location
    """

    a,b = location
    inLocation = False

    l = []
    for interval in intervals:
        if a >= interval[0] and a <= interval[1]:
            inLocation = True

        if inLocation:
            x = min(b, interval[1]) + 1 - max(a, interval[0])
            l.append( (interval, x / float(b-a+1) ) )

        if b <= interval[1]:
            break

    return l


class Interval:
    def __init__(self, a, b):
        self.a = a
        self.b = b

    def __in__(self, v):
        """
        return True if value v is in this interval
        """
        return x >= self.a and x <= self.b
        
    def tuple(self):
        return (self.a, self.b)
    


#-------------------------------------------------------------------------------
# CODE
