####################
# version 2.0
####################


"""
Interfaces for Background Model
for hit_scanner and cluster_scanner

I_BG
 - matrix
 - parameters
 -* location
 -* organism
 -* gc range


 *: comme from training sequences

I_BGDB
database of BG

I_TFBSscanner

get(sequence, matrix) -> BG object

"""
from numpy import zeros

import Core.types as types
import Core.Types.tab


class I_BGDB(object):
    """
    interface for databases of background model
    """

    def __init__(self):
        pass


    def get(self, sequence, matrix):
        """
        return BG object corresponding to sequence and matrix
        """
        raise NotImplementedError


class I_BG(object):
    """
    interface for background model 
    """
    def __init__(self, parameters):
        self.parameters = parameters
    
    def get_parameters(self):
        return self.parameters


    def get_p(self):
        return self.p

    def compute_p(self, matrix, sequences, Pataud, windowWidth=400):
        """
        compute a background model by scanning sequences with a threshold computed in Pataud.build
        
        matrix      -- a pwm matrix
        
        sequences   -- dna sequences
        
        Pataud      -- a object to scan the sequences
        
        windowWidth -- a width for the smoothing window
        """
        SMOOTHING_WINDOW_WIDTH = 500

        S = len(sequences)
        L = len(sequences[0])
        W = windowWidth
        M = len(matrix)

        STEP = W

        hitCount = zeros(L, 'i')

        # build hit count tab
        l = Pataud.scan(sequences, matrix, self)
        #print l
        for s in l:
            for i in s:
                hitCount[i[0]] += 1
        #
        p = zeros( L, 'f')
        for I in range(0, L-W+1-M+1, STEP):
            #print sum(hitCount[I:I+STEP])
            x = sum(hitCount[I:I+STEP]) / (float(STEP) * S)
            #print I, I+STEP, x
            p[I:I+STEP] = x
            

        p[I+STEP:] = x
        #print p
        #print p
        self.p = Core.Types.tab.smooth(p, SMOOTHING_WINDOW_WIDTH)



class I_TFBSscanner(object):
    """Interface for all TFBS Scanner

    """
    
    def __init__(self, bg=None):
        """
        bg -- object that implements I_BG
        """
        self.bg = bg

    def find(self, sequences, BSmodels=None, bg=None):
        """
        Find TFBSs for all BS models
        
        sequences -- list of objects (Sequence)
        
        BSmodels  -- list of models

        BG        -- object that implements I_BG
        
        return list(matrix) of list(sequences) of tuples (pos, strand, score, ...) corresponding to hits
        """
        raise NotImplementedError
        
        

