#! /usr/bin/env python

###################
# version 2.0
###################

#############################################################################################
# fasta2dump
# Program that takes a multifasta containing promoter sequences for an organism, and create 
# a serialized object saved into a binary file. The object is a list of sequences object
# from the class Sequence in Bio.sequence
#
# author: Laurie Tonon
# date: 28/10/2009
#
#############################################################################################

import os,sys

# export path to find the modules
PYTHONPATH = [['TFME'], ['TFME', 'MLib.zlib']]
if sys.version_info < (2, 3): print >> sys.stderr, 'python >= 2.3 is required!'; sys.exit()
[sys.path.insert(1, os.path.join(sys.path[0], *path)) for path in PYTHONPATH]


import imp

#********************************************************************
# Check dependencies
#********************************************************************
try:
    import numpy
    version= numpy.__version__

except ImportError:
    print 'WARNING: TFM-Explorer requires Numpy '
    sys.exit(1)
    
try:
    import Pyrex

except ImportError:
    print 'WARNING: TFM-Explorer requires Pyrex '
    sys.exit(1)



import optparse
import Core.UI.cli as cli
import Bio.sequence

usage = """
       python %prog  --input=filename.fasta  --output=dumpfilename.seq.db --organism=organism  --location=x:x
"""

parser = optparse.OptionParser(usage)

parser.add_option("-l", "--location", dest="location", help="location of the extracted sequences relative to TSS (example -2000:-1)", action="store", type="string", default='')
parser.add_option("-i", "--input", dest="input", help="input fasta filename", action="store", default="")
parser.add_option("-o", "--output", dest="output", action="store", type="string", default="", help="output file created with the promoter sequences")
parser.add_option("--organism", dest="organism", help="organism corresponding to the sequences", action="store", type="string", default="")
parser.add_option("--debug", dest="debug", action="store_true", default=False,  help="show program errors. For developpers only")
parser.add_option("-v", "--verbosity", action="store", dest="verbosity", type="int", default=1,  help="display complete process on screen (between 1 and 3) (default=1)")

(options, args) = parser.parse_args()
cli.VERBOSITY = options.verbosity

def run(args, options):

    #**********************************************
    # Check user inputs
    #**********************************************

    if not options.output:
        cli.error("No output file provided. Cannot continue.")
        sys.exit(1)
                
    elif os.path.dirname(options.output) != '' and not os.path.isdir(os.path.dirname(options.output)): # check if the output directory exists
        cli.error('The output directory does not exists')
        sys.exit(1)
    else:
        if os.path.isfile(options.output): # if a dump file with the same name already exists, try to replace it
            try:
                os.remove(options.output)
            except OSError:
                cli.error("A dump file with the same name as "+options.output+" already exists and cannot be removed. Please delete or rename this file and lauch fasta2dump.py again.")
                sys.exit(1)
        elif os.path.isfile(options.output+'.db'):
            try:
                os.remove(options.output+'.db')
            except OSError:
                cli.error("A dump file with the same name as "+options.output+'.db'+" already exists and cannot be removed. Please delete or rename this file and lauch fasta2dump.py again.")
                sys.exit(1)
            
    if options.location:
        location = Bio.sequence.str2location(options.location) # if no location specified, a default one is used in the creation of the sequence
        
        if location[0] == location[1]:
            cli.error("The location is null")
            sys.exit(1)
    else:
        location="";
        cli.warning("No location was provided for the sequences, the default value is 0:length of the sequence")
		
    if not options.input:
        cli.error("No input file provided. Cannot continue.")
        sys.exit(1)
    elif not os.path.isfile(options.input):
        cli.error("Cannot find the fasta file.")
        sys.exit(1)
    
    if not options.organism:
        cli.error("The organism of the sequences is not provided. Cannot continue")
        sys.exit(1)

    #******************************************
	# Call the method to create the fasta file
	#******************************************
    Bio.sequence.fasta2dump(options.input,options.organism,location,options.output)

    print "TFM-Explorer fasta to dump converter job successfully ended"


if __name__ == '__main__':
    try:
        run(args, options)
    except:
        cli.error('Error while running fasta2dump')
        if options.debug:
            raise
